@@ -221,6 +221,13 @@ class ProxyConfiguration(object):
     def base_config(self):
         return self.globals.base_config
 
+    def config_files(self):
+        """
+        Returns a dictionary with all configuration filenames and there timestamps.
+        Contains any included files as well (see `base` option).
+        """
+        return self.configuration.get('__config_files__', {})
+
 def list_of_dicts_to_ordered_dict(dictlist):
     """
     >>> d = list_of_dicts_to_ordered_dict([{'a': 1}, {'b': 2}, {'c': 3}])
@@ -1168,7 +1175,15 @@ class CacheConfiguration(ConfigurationBase):
 
             tile_creator_class = None
 
-            if not self.context.renderd and renderd_address:
+            use_renderd = bool(renderd_address)
+            if self.context.renderd:
+                # we _are_ renderd
+                use_renderd = False
+            if self.conf.get('disable_storage', False):
+                # can't ask renderd to create tiles that shouldn't be cached
+                use_renderd = False
+
+            if use_renderd:
                 from mapproxy.cache.renderd import RenderdTileCreator, has_renderd_support
                 if not has_renderd_support():
                     raise ConfigurationError("renderd requires Python >=2.6 and requests")
@@ -1389,6 +1404,22 @@ def fi_xslt_transformers(conf, context):
             fi_transformers[info_type] = XSLTransformer(fi_xslt)
     return fi_transformers
 
+def extents_for_srs(bbox_srs):
+    from mapproxy.layer import DefaultMapExtent, MapExtent
+    from mapproxy.srs import SRS
+    extents = {}
+    for srs in bbox_srs:
+        if isinstance(srs, str):
+            bbox = DefaultMapExtent()
+        else:
+            srs, bbox = srs['srs'], srs['bbox']
+            bbox = MapExtent(bbox, SRS(srs))
+
+        extents[srs] = bbox
+
+    return extents
+
+
 class ServiceConfiguration(ConfigurationBase):
     def services(self):
         services = []
@@ -1483,6 +1514,7 @@ class ServiceConfiguration(ConfigurationBase):
 
     def wms_service(self, conf):
         from mapproxy.service.wms import WMSServer
+        from mapproxy.request.wms import Version
 
         md = conf.get('md', {})
         tile_layers = self.tile_layers(conf)
@@ -1499,7 +1531,7 @@ class ServiceConfiguration(ConfigurationBase):
             global_key='wms.concurrent_layer_renderer')
         image_formats_names = self.context.globals.get_value('image_formats', conf,
                                                        global_key='wms.image_formats')
-        image_formats = {}
+        image_formats = odict()
         for format in image_formats_names:
             opts = self.context.globals.image_options.image_opts({}, format)
             if opts.format in image_formats:
@@ -1509,7 +1541,11 @@ class ServiceConfiguration(ConfigurationBase):
         info_types = conf.get('featureinfo_types')
         srs = self.context.globals.get_value('srs', conf, global_key='wms.srs')
         self.context.globals.base_config.wms.srs = srs
-        bbox_srs = conf.get('bbox_srs')
+        srs_extents = extents_for_srs(conf.get('bbox_srs', []))
+
+        versions = conf.get('versions')
+        if versions:
+            versions = sorted([Version(v) for v in versions])
 
         max_output_pixels = self.context.globals.get_value('max_output_pixels', conf,
             global_key='wms.max_output_pixels')
@@ -1523,8 +1559,8 @@ class ServiceConfiguration(ConfigurationBase):
             image_formats=image_formats, info_types=info_types,
             srs=srs, tile_layers=tile_layers, strict=strict, on_error=on_source_errors,
             concurrent_layer_renderer=concurrent_layer_renderer,
-            max_output_pixels=max_output_pixels, bbox_srs=bbox_srs,
-            max_tile_age=max_tile_age)
+            max_output_pixels=max_output_pixels, srs_extents=srs_extents,
+            max_tile_age=max_tile_age, versions=versions)
 
         server.fi_transformers = fi_xslt_transformers(conf, self.context)
 
@@ -1573,11 +1609,15 @@ def load_configuration_file(files, working_dir):
     """
     Return configuration dict from imported files
     """
-    conf_dict = {}
+    # record all config files with timestamp for reloading
+    conf_dict = {'__config_files__': {}}
     for conf_file in files:
         conf_file = os.path.normpath(os.path.join(working_dir, conf_file))
         log.info('reading: %s' % conf_file)
         current_dict = load_yaml_file(conf_file)
+
+        conf_dict['__config_files__'][os.path.abspath(conf_file)] = os.path.getmtime(conf_file)
+
         if 'base' in current_dict:
             current_working_dir = os.path.dirname(conf_file)
             base_files = current_dict.pop('base')
