@@ -28,6 +28,22 @@ import logging
 log = logging.getLogger('mapproxy.image')
 
 
+magic_bytes = [
+    ('png', ("\211PNG\r\n\032\n",)),
+    ('jpeg', ("\xFF\xD8",)),
+    ('tiff', ("MM\x00\x2a", "II\x2a\x00",)),
+    ('gif', ("GIF87a", "GIF89a",)),
+]
+
+def peek_image_format(buf):
+    buf.seek(0)
+    header = buf.read(10)
+    buf.seek(0)
+    for format, bytes in magic_bytes:
+        if header.startswith(bytes):
+            return format
+    return None
+
 class ImageSource(object):
     """
     This class wraps either a PIL image, a file-like object, or a file name.
@@ -72,6 +88,9 @@ class ImageSource(object):
             except IOError:
                 pass
 
+        if self._img:
+            self._img = None
+
     @property
     def filename(self):
         return self._fname
@@ -108,7 +127,7 @@ class ImageSource(object):
         if not self._buf and self._fname:
             self._buf = open(self._fname, 'rb')
         elif not hasattr(self._buf, 'seek'):
-            if isinstance(self._buf, ReadBufWrapper):
+            if not isinstance(self._buf, ReadBufWrapper):
                 self._buf = ReadBufWrapper(self._buf)
         else:
             self._buf.seek(0)
@@ -131,6 +150,10 @@ class ImageSource(object):
             self._buf = img_to_buf(self._img, image_opts=image_opts)
         else:
             self._make_seekable_buf() if seekable else self._make_readable_buf()
+            if self.image_opts and image_opts and not self.image_opts.format and image_opts.format:
+                # need actual image_opts.format for next check
+                self.image_opts = self.image_opts.copy()
+                self.image_opts.format = peek_image_format(self._buf)
             if self.image_opts and image_opts and self.image_opts.format != image_opts.format:
                 log.debug('converting image from %s -> %s' % (self.image_opts, image_opts))
                 self.source = self.as_image()
