@@ -23,19 +23,24 @@ import operator
 from mapproxy.config import abspath
 from mapproxy.config.loader import ConfigurationError
 from mapproxy.config.coverage import load_coverage
-from mapproxy.srs import SRS
+from mapproxy.srs import SRS, TransformationError
 from mapproxy.util.py import memoize
 from mapproxy.util.times import timestamp_from_isodate, timestamp_before
 from mapproxy.util.coverage import MultiCoverage, BBOXCoverage, GeomCoverage
+from mapproxy.util.geom import GeometryError, EmptyGeometryError
 from mapproxy.util.yaml import load_yaml_file, YAMLError
 from mapproxy.seed.util import bidict
 from mapproxy.seed.seeder import SeedTask, CleanupTask
 from mapproxy.seed.spec import validate_seed_conf
+from mapproxy.util.ogr import OGRShapeReaderError
 
 
 class SeedConfigurationError(ConfigurationError):
     pass
 
+class EmptyCoverageError(Exception):
+    pass
+
 
 import logging
 log = logging.getLogger('mapproxy.seed.config')
@@ -143,7 +148,19 @@ class SeedingConfiguration(object):
             raise SeedConfigurationError('coverage %s not found. available coverages: %s' % (
                 name, ','.join((self.conf.get('coverages') or {}).keys())))
 
-        return load_coverage(coverage_conf)
+        try:
+            coverage = load_coverage(coverage_conf)
+        except OGRShapeReaderError, ex:
+            raise SeedConfigurationError("can't load coverage '%s'. %s" % (name, ex))
+        except GeometryError, ex:
+            raise SeedConfigurationError("invalid geometry in coverage '%s'. %s" % (name, ex))
+        except EmptyGeometryError, ex:
+            raise EmptyCoverageError("coverage '%s' contains no geometries. %s" % (name, ex))
+
+        # without extend we have an empty coverage
+        if not coverage.extent.llbbox:
+            raise EmptyCoverageError("coverage '%s' contains no geometries." % name)
+        return coverage
 
     def cache(self, cache_name):
         cache = {}
@@ -156,13 +173,13 @@ class SeedingConfiguration(object):
         return cache
 
     def seed_tasks_names(self):
-        seeds = self.conf.get('seeds', {})
+        seeds = self.conf.get('seeds') or {}
         if seeds:
             return seeds.keys()
         return []
 
     def cleanup_tasks_names(self):
-        cleanups = self.conf.get('cleanups', {})
+        cleanups = self.conf.get('cleanups') or {}
         if cleanups:
             return cleanups.keys()
         return []
@@ -172,8 +189,11 @@ class SeedingConfiguration(object):
         Return seed tasks.
         """
         tasks = []
-        for seed_name, seed_conf in self.conf.get('seeds', {}).iteritems():
-            if names is not None and seed_name not in names: continue
+        if names is None:
+            names = (self.conf.get('seeds') or {}).keys()
+
+        for seed_name in names:
+            seed_conf = self.conf['seeds'][seed_name]
             seed_conf = SeedConfiguration(seed_name, seed_conf, self)
             for task in seed_conf.seed_tasks():
                 tasks.append(task)
@@ -184,8 +204,11 @@ class SeedingConfiguration(object):
         Return cleanup tasks.
         """
         tasks = []
-        for cleanup_name, cleanup_conf in self.conf.get('cleanups', {}).iteritems():
-            if names is not None and cleanup_name not in names: continue
+        if names is None:
+            names = (self.conf.get('cleanups') or {}).keys()
+
+        for cleanup_name in names:
+            cleanup_conf = self.conf['cleanups'][cleanup_name]
             cleanup_conf = CleanupConfiguration(cleanup_name, cleanup_conf, self)
             for task in cleanup_conf.cleanup_tasks():
                 tasks.append(task)
@@ -206,7 +229,10 @@ class ConfigurationBase(object):
     def _coverages(self):
         coverage = None
         if 'coverages' in self.conf:
-            coverages = [self.seeding_conf.coverage(c) for c in self.conf.get('coverages', {})]
+            try:
+                coverages = [self.seeding_conf.coverage(c) for c in self.conf.get('coverages', {})]
+            except EmptyCoverageError:
+                return False
             if len(coverages) == 1:
                 coverage = coverages[0]
             else:
@@ -260,12 +286,19 @@ class SeedConfiguration(ConfigurationBase):
             for cache_name, cache in self.caches.iteritems():
                 tile_manager = cache[grid_name]
                 grid = self.seeding_conf.grids[grid_name]
-                if self.coverage:
-                    if isinstance(self.coverage, GeomCoverage) and self.coverage.geom.is_empty:
-                        continue
+                if self.coverage is False:
+                    coverage = False
+                elif self.coverage:
                     coverage = self.coverage.transform_to(grid.srs)
                 else:
                     coverage = BBOXCoverage(grid.bbox, grid.srs)
+
+                try:
+                    if coverage is not False:
+                        coverage.extent.llbbox
+                except TransformationError:
+                    raise SeedConfigurationError('%s: coverage transformation error' % self.name)
+
                 if self.levels:
                     levels = self.levels.for_grid(grid)
                 else:
@@ -298,12 +331,22 @@ class CleanupConfiguration(ConfigurationBase):
             for cache_name, cache in self.caches.iteritems():
                 tile_manager = cache[grid_name]
                 grid = self.seeding_conf.grids[grid_name]
-                if self.coverage:
+                if self.coverage is False:
+                    coverage = False
+                    complete_extent = False
+                elif self.coverage:
                     coverage = self.coverage.transform_to(grid.srs)
                     complete_extent = False
                 else:
                     coverage = BBOXCoverage(grid.bbox, grid.srs)
                     complete_extent = True
+
+                try:
+                    if coverage is not False:
+                        coverage.extent.llbbox
+                except TransformationError:
+                    raise SeedConfigurationError('%s: coverage transformation error' % self.name)
+
                 if self.levels:
                     levels = self.levels.for_grid(grid)
                 else:
@@ -311,7 +354,7 @@ class CleanupConfiguration(ConfigurationBase):
 
                 if not tile_manager.cache.supports_timestamp:
                     # for caches without timestamp support (like MBTiles)
-                    if self.remove_timestamp is self.init_time:
+                    if self.remove_timestamp is self.init_time or self.remove_timestamp == 0:
                         # remove everything
                         self.remove_timestamp = 0
                     else:
